/*
 * Copyright (c) 2002 Sandro Sigala.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef MAP_H
#define MAP_H

#include <vector>

#include <fxdefs.h>
#include <FXString.h>

#include "fxreptoned.h"

struct Transporter {
	int orig_x, orig_y;
	int dest_x, dest_y;

	Transporter(int ox, int oy, int dx, int dy)
		: orig_x(ox), orig_y(oy), dest_x(dx), dest_y(dy) {}
};

struct Map {
	int width, height;
	FXString password;
	FXString comments;
	vector<Transporter *> transporters;
	int time_limit;
	bool ismodified;
	bool readonly;

	Map();
	~Map();

	void clear();

	Sprite xy(int x, int y) const {
		return *(data + x + y*width);
	}
	Sprite& xy(int x, int y) {
		return *(data + x + y*width);
	}

	void copy(const Map *src);
	void newMap(int w, int h);
	void newDummyMap(int w, int h);
	void resize(int w, int h, bool top, bool left);
	void load(const FXString& filename);
	void save(const FXString& filename);
	bool isModified() const;
	int count(Sprite s) const;

private:
	Sprite *data;
	void addTransporter(int ox, int oy, int dx, int dy);
	Transporter *findTransporter(int x, int y);
	void deleteTransporters();
	void generatePassword();
};

struct Episode {
	FXString package;
	FXString name;
	FXString path;
	FXString author;
	FXString comments;
	vector<Map *> maps;
	bool isdataloaded;
	bool ismodified;
	bool isbroken;

	Episode();
	~Episode();
	void clear();
	void appendMap(Map *map);
	void load(const FXString& epath);
	void save(const FXString& epath);
	bool isModified() const;

	static void loadEpisodesList(vector<Episode>& v, const FXString& epath);
};

#endif // !MAP_H

